# bash completion for skydive                              -*- shell-script -*-

__debug()
{
    if [[ -n ${BASH_COMP_DEBUG_FILE} ]]; then
        echo "$*" >> "${BASH_COMP_DEBUG_FILE}"
    fi
}

# Homebrew on Macs have version 1.3 of bash-completion which doesn't include
# _init_completion. This is a very minimal version of that function.
__my_init_completion()
{
    COMPREPLY=()
    _get_comp_words_by_ref "$@" cur prev words cword
}

__index_of_word()
{
    local w word=$1
    shift
    index=0
    for w in "$@"; do
        [[ $w = "$word" ]] && return
        index=$((index+1))
    done
    index=-1
}

__contains_word()
{
    local w word=$1; shift
    for w in "$@"; do
        [[ $w = "$word" ]] && return
    done
    return 1
}

__handle_reply()
{
    __debug "${FUNCNAME[0]}"
    case $cur in
        -*)
            if [[ $(type -t compopt) = "builtin" ]]; then
                compopt -o nospace
            fi
            local allflags
            if [ ${#must_have_one_flag[@]} -ne 0 ]; then
                allflags=("${must_have_one_flag[@]}")
            else
                allflags=("${flags[*]} ${two_word_flags[*]}")
            fi
            COMPREPLY=( $(compgen -W "${allflags[*]}" -- "$cur") )
            if [[ $(type -t compopt) = "builtin" ]]; then
                [[ "${COMPREPLY[0]}" == *= ]] || compopt +o nospace
            fi

            # complete after --flag=abc
            if [[ $cur == *=* ]]; then
                if [[ $(type -t compopt) = "builtin" ]]; then
                    compopt +o nospace
                fi

                local index flag
                flag="${cur%%=*}"
                __index_of_word "${flag}" "${flags_with_completion[@]}"
                if [[ ${index} -ge 0 ]]; then
                    COMPREPLY=()
                    PREFIX=""
                    cur="${cur#*=}"
                    ${flags_completion[${index}]}
                    if [ -n "${ZSH_VERSION}" ]; then
                        # zfs completion needs --flag= prefix
                        eval "COMPREPLY=( \"\${COMPREPLY[@]/#/${flag}=}\" )"
                    fi
                fi
            fi
            return 0;
            ;;
    esac

    # check if we are handling a flag with special work handling
    local index
    __index_of_word "${prev}" "${flags_with_completion[@]}"
    if [[ ${index} -ge 0 ]]; then
        ${flags_completion[${index}]}
        return
    fi

    # we are parsing a flag and don't have a special handler, no completion
    if [[ ${cur} != "${words[cword]}" ]]; then
        return
    fi

    local completions
    completions=("${commands[@]}")
    if [[ ${#must_have_one_noun[@]} -ne 0 ]]; then
        completions=("${must_have_one_noun[@]}")
    fi
    if [[ ${#must_have_one_flag[@]} -ne 0 ]]; then
        completions+=("${must_have_one_flag[@]}")
    fi
    COMPREPLY=( $(compgen -W "${completions[*]}" -- "$cur") )

    if [[ ${#COMPREPLY[@]} -eq 0 && ${#noun_aliases[@]} -gt 0 && ${#must_have_one_noun[@]} -ne 0 ]]; then
        COMPREPLY=( $(compgen -W "${noun_aliases[*]}" -- "$cur") )
    fi

    if [[ ${#COMPREPLY[@]} -eq 0 ]]; then
        declare -F __custom_func >/dev/null && __custom_func
    fi

    __ltrim_colon_completions "$cur"
}

# The arguments should be in the form "ext1|ext2|extn"
__handle_filename_extension_flag()
{
    local ext="$1"
    _filedir "@(${ext})"
}

__handle_subdirs_in_dir_flag()
{
    local dir="$1"
    pushd "${dir}" >/dev/null 2>&1 && _filedir -d && popd >/dev/null 2>&1
}

__handle_flag()
{
    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    # if a command required a flag, and we found it, unset must_have_one_flag()
    local flagname=${words[c]}
    local flagvalue
    # if the word contained an =
    if [[ ${words[c]} == *"="* ]]; then
        flagvalue=${flagname#*=} # take in as flagvalue after the =
        flagname=${flagname%%=*} # strip everything after the =
        flagname="${flagname}=" # but put the = back
    fi
    __debug "${FUNCNAME[0]}: looking for ${flagname}"
    if __contains_word "${flagname}" "${must_have_one_flag[@]}"; then
        must_have_one_flag=()
    fi

    # if you set a flag which only applies to this command, don't show subcommands
    if __contains_word "${flagname}" "${local_nonpersistent_flags[@]}"; then
      commands=()
    fi

    # keep flag value with flagname as flaghash
    if [ -n "${flagvalue}" ] ; then
        flaghash[${flagname}]=${flagvalue}
    elif [ -n "${words[ $((c+1)) ]}" ] ; then
        flaghash[${flagname}]=${words[ $((c+1)) ]}
    else
        flaghash[${flagname}]="true" # pad "true" for bool flag
    fi

    # skip the argument to a two word flag
    if __contains_word "${words[c]}" "${two_word_flags[@]}"; then
        c=$((c+1))
        # if we are looking for a flags value, don't show commands
        if [[ $c -eq $cword ]]; then
            commands=()
        fi
    fi

    c=$((c+1))

}

__handle_noun()
{
    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    if __contains_word "${words[c]}" "${must_have_one_noun[@]}"; then
        must_have_one_noun=()
    elif __contains_word "${words[c]}" "${noun_aliases[@]}"; then
        must_have_one_noun=()
    fi

    nouns+=("${words[c]}")
    c=$((c+1))
}

__handle_command()
{
    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    local next_command
    if [[ -n ${last_command} ]]; then
        next_command="_${last_command}_${words[c]//:/__}"
    else
        if [[ $c -eq 0 ]]; then
            next_command="_$(basename "${words[c]//:/__}")"
        else
            next_command="_${words[c]//:/__}"
        fi
    fi
    c=$((c+1))
    __debug "${FUNCNAME[0]}: looking for ${next_command}"
    declare -F $next_command >/dev/null && $next_command
}

__handle_word()
{
    if [[ $c -ge $cword ]]; then
        __handle_reply
        return
    fi
    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
    if [[ "${words[c]}" == -* ]]; then
        __handle_flag
    elif __contains_word "${words[c]}" "${commands[@]}"; then
        __handle_command
    elif [[ $c -eq 0 ]] && __contains_word "$(basename "${words[c]}")" "${commands[@]}"; then
        __handle_command
    else
        __handle_noun
    fi
    __handle_word
}

_skydive_agent()
{
    last_command="skydive_agent"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--host-id=")
    local_nonpersistent_flags+=("--host-id=")
    flags+=("--listen=")
    local_nonpersistent_flags+=("--listen=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_allinone()
{
    last_command="skydive_allinone"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer-uid=")
    local_nonpersistent_flags+=("--analyzer-uid=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_analyzer()
{
    last_command="skydive_analyzer"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--listen=")
    local_nonpersistent_flags+=("--listen=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_bash-completion()
{
    last_command="skydive_bash-completion"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--help")
    flags+=("-h")
    local_nonpersistent_flags+=("--help")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_alert_create()
{
    last_command="skydive_client_alert_create"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--action=")
    local_nonpersistent_flags+=("--action=")
    flags+=("--description=")
    local_nonpersistent_flags+=("--description=")
    flags+=("--expression=")
    local_nonpersistent_flags+=("--expression=")
    flags+=("--name=")
    local_nonpersistent_flags+=("--name=")
    flags+=("--trigger=")
    local_nonpersistent_flags+=("--trigger=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_alert_delete()
{
    last_command="skydive_client_alert_delete"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_alert_get()
{
    last_command="skydive_client_alert_get"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_alert_list()
{
    last_command="skydive_client_alert_list"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_alert()
{
    last_command="skydive_client_alert"
    commands=()
    commands+=("create")
    commands+=("delete")
    commands+=("get")
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_capture_create()
{
    last_command="skydive_client_capture_create"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--bpf=")
    local_nonpersistent_flags+=("--bpf=")
    flags+=("--description=")
    local_nonpersistent_flags+=("--description=")
    flags+=("--extra-tcp-metric")
    local_nonpersistent_flags+=("--extra-tcp-metric")
    flags+=("--gremlin=")
    local_nonpersistent_flags+=("--gremlin=")
    flags+=("--header-size=")
    local_nonpersistent_flags+=("--header-size=")
    flags+=("--ip-defrag")
    local_nonpersistent_flags+=("--ip-defrag")
    flags+=("--layer-key-mode=")
    local_nonpersistent_flags+=("--layer-key-mode=")
    flags+=("--name=")
    local_nonpersistent_flags+=("--name=")
    flags+=("--node=")
    local_nonpersistent_flags+=("--node=")
    flags+=("--port=")
    local_nonpersistent_flags+=("--port=")
    flags+=("--rawpacket-limit=")
    local_nonpersistent_flags+=("--rawpacket-limit=")
    flags+=("--reassamble-tcp")
    local_nonpersistent_flags+=("--reassamble-tcp")
    flags+=("--type=")
    local_nonpersistent_flags+=("--type=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_capture_delete()
{
    last_command="skydive_client_capture_delete"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_capture_get()
{
    last_command="skydive_client_capture_get"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_capture_list()
{
    last_command="skydive_client_capture_list"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_capture()
{
    last_command="skydive_client_capture"
    commands=()
    commands+=("create")
    commands+=("delete")
    commands+=("get")
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_inject-packet_create()
{
    last_command="skydive_client_inject-packet_create"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--count=")
    local_nonpersistent_flags+=("--count=")
    flags+=("--dst=")
    local_nonpersistent_flags+=("--dst=")
    flags+=("--dstIP=")
    local_nonpersistent_flags+=("--dstIP=")
    flags+=("--dstMAC=")
    local_nonpersistent_flags+=("--dstMAC=")
    flags+=("--dstPort=")
    local_nonpersistent_flags+=("--dstPort=")
    flags+=("--id=")
    local_nonpersistent_flags+=("--id=")
    flags+=("--increment")
    local_nonpersistent_flags+=("--increment")
    flags+=("--interval=")
    local_nonpersistent_flags+=("--interval=")
    flags+=("--payload=")
    local_nonpersistent_flags+=("--payload=")
    flags+=("--src=")
    local_nonpersistent_flags+=("--src=")
    flags+=("--srcIP=")
    local_nonpersistent_flags+=("--srcIP=")
    flags+=("--srcMAC=")
    local_nonpersistent_flags+=("--srcMAC=")
    flags+=("--srcPort=")
    local_nonpersistent_flags+=("--srcPort=")
    flags+=("--type=")
    local_nonpersistent_flags+=("--type=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_inject-packet_delete()
{
    last_command="skydive_client_inject-packet_delete"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_inject-packet_get()
{
    last_command="skydive_client_inject-packet_get"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_inject-packet_list()
{
    last_command="skydive_client_inject-packet_list"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_inject-packet()
{
    last_command="skydive_client_inject-packet"
    commands=()
    commands+=("create")
    commands+=("delete")
    commands+=("get")
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_pcap()
{
    last_command="skydive_client_pcap"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--trace=")
    two_word_flags+=("-t")
    local_nonpersistent_flags+=("--trace=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_query()
{
    last_command="skydive_client_query"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--format=")
    local_nonpersistent_flags+=("--format=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_shell()
{
    last_command="skydive_client_shell"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--script=")
    local_nonpersistent_flags+=("--script=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_status()
{
    last_command="skydive_client_status"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_topology_query()
{
    last_command="skydive_client_topology_query"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--format=")
    local_nonpersistent_flags+=("--format=")
    flags+=("--gremlin=")
    local_nonpersistent_flags+=("--gremlin=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_topology()
{
    last_command="skydive_client_topology"
    commands=()
    commands+=("query")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_user-metadata_create()
{
    last_command="skydive_client_user-metadata_create"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--gremlin=")
    local_nonpersistent_flags+=("--gremlin=")
    flags+=("--key=")
    local_nonpersistent_flags+=("--key=")
    flags+=("--value=")
    local_nonpersistent_flags+=("--value=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_user-metadata_delete()
{
    last_command="skydive_client_user-metadata_delete"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_user-metadata_list()
{
    last_command="skydive_client_user-metadata_list"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_user-metadata()
{
    last_command="skydive_client_user-metadata"
    commands=()
    commands+=("create")
    commands+=("delete")
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_workflow_call()
{
    last_command="skydive_client_workflow_call"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_workflow_create()
{
    last_command="skydive_client_workflow_create"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--path=")
    local_nonpersistent_flags+=("--path=")
    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_workflow_delete()
{
    last_command="skydive_client_workflow_delete"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_workflow_list()
{
    last_command="skydive_client_workflow_list"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client_workflow()
{
    last_command="skydive_client_workflow"
    commands=()
    commands+=("call")
    commands+=("create")
    commands+=("delete")
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")
    flags+=("--password=")
    flags+=("--username=")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_client()
{
    last_command="skydive_client"
    commands=()
    commands+=("alert")
    commands+=("capture")
    commands+=("inject-packet")
    commands+=("pcap")
    commands+=("query")
    commands+=("shell")
    commands+=("status")
    commands+=("topology")
    commands+=("user-metadata")
    commands+=("workflow")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--analyzer=")
    flags+=("--password=")
    flags+=("--username=")
    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive_version()
{
    last_command="skydive_version"
    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_skydive()
{
    last_command="skydive"
    commands=()
    commands+=("agent")
    commands+=("allinone")
    commands+=("analyzer")
    commands+=("bash-completion")
    commands+=("client")
    commands+=("version")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--conf=")
    two_word_flags+=("-c")
    flags+=("--config-backend=")
    two_word_flags+=("-b")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

__start_skydive()
{
    local cur prev words cword
    declare -A flaghash 2>/dev/null || :
    if declare -F _init_completion >/dev/null 2>&1; then
        _init_completion -s || return
    else
        __my_init_completion -n "=" || return
    fi

    local c=0
    local flags=()
    local two_word_flags=()
    local local_nonpersistent_flags=()
    local flags_with_completion=()
    local flags_completion=()
    local commands=("skydive")
    local must_have_one_flag=()
    local must_have_one_noun=()
    local last_command
    local nouns=()

    __handle_word
}

if [[ $(type -t compopt) = "builtin" ]]; then
    complete -o default -F __start_skydive skydive
else
    complete -o default -o nospace -F __start_skydive skydive
fi

# ex: ts=4 sw=4 et filetype=sh
